@@ -94,10 +94,6 @@ static const GtkTargetEntry target_table[] = {
 	{ "_NETSCAPE_URL", 0, 1 },
 };
 
-static const GtkTargetEntry source_table[] = {
-	{ "text/uri-list", 0, 0 },
-};
-
 static gboolean totem_action_open_files (Totem *totem, char **list);
 static gboolean totem_action_open_files_list (Totem *totem, GSList *list);
 static void update_fullscreen_size (Totem *totem);
@@ -199,6 +195,12 @@ totem_action_exit (Totem *totem)
 	if (totem == NULL)
 		exit (0);
 
+#ifdef HAVE_REMOTE
+	if (totem->remote != NULL) {
+		g_object_unref (G_OBJECT (totem->remote));
+	}
+#endif /* HAVE_REMOTE */
+
 	if (totem->win != NULL) {
 		gtk_widget_hide (totem->win);
 		display = gtk_widget_get_display (totem->win);
@@ -344,7 +346,8 @@ totem_action_eject (Totem *totem)
 void
 totem_action_show_properties (Totem *totem)
 {
-	totem_sidebar_set_current_page (totem, "properties");
+        if (totem_is_fullscreen (totem) == FALSE)
+		totem_sidebar_set_current_page (totem, "properties");
 }
 
 void
@@ -786,12 +789,15 @@ totem_action_open_location (Totem *totem)
 
 		for (p = recent_items; p != NULL; p = p->next)
 		{
-			if (!gtk_recent_info_has_group ((GtkRecentInfo *) p->data, "TotemStreams"))
+			GtkRecentInfo *info = (GtkRecentInfo *) p->data;
+			if (!gtk_recent_info_has_group (info, "TotemStreams")) {
+				gtk_recent_info_unref (info);
 				continue;
+			}
 
 			gtk_list_store_append (GTK_LIST_STORE (model), &iter);
-			gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, gtk_recent_info_get_uri ((GtkRecentInfo *) p->data), -1);
-			gtk_recent_info_unref ((GtkRecentInfo *) p->data);
+			gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, gtk_recent_info_get_uri (info), -1);
+			gtk_recent_info_unref (info);
 		}
 	}
 
@@ -2209,6 +2215,8 @@ playlist_changed_cb (GtkWidget *playlist, Totem *totem)
 	if (mrl == NULL)
 		return;
 
+	//FIXME we shouldn't compare the 2 URLs
+	// http://bugzilla.gnome.org/show_bug.cgi?id=364311
 	if (totem->mrl == NULL
 			|| (totem->mrl != NULL && mrl != NULL
 			&& strcmp (totem->mrl, mrl) != 0))
@@ -2359,6 +2367,19 @@ window_realize_cb (GtkWidget *widget, Totem *totem)
 			"changed", G_CALLBACK (theme_changed_cb), totem);
 }
 
+static void
+window_unrealize_cb (GtkWidget *widget, Totem *totem)
+{
+	GdkScreen *screen;
+
+	screen = gtk_widget_get_screen (widget);
+	g_signal_handlers_disconnect_by_func
+		(screen, G_CALLBACK (size_changed_cb), totem);
+	g_signal_handlers_disconnect_by_func
+		(gtk_icon_theme_get_for_screen (screen),
+		 G_CALLBACK (theme_changed_cb), totem);
+}
+
 static void
 popup_timeout_add(Totem* totem)
 {
@@ -2609,7 +2630,6 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 		break;
 #ifdef HAVE_XFREE
 	case XF86XK_AudioPlay:
-	case XF86XK_AudioPause:
 #endif /* HAVE_XFREE */
 	case GDK_p:
 	case GDK_P:
@@ -2618,6 +2638,12 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 		else 
 			totem_action_play_pause (totem);
 		break;
+#ifdef HAVE_XFREE
+	case XF86XK_AudioPause:
+	case XF86XK_AudioStop:
+		totem_action_pause (totem);
+		break;
+#endif /* HAVE_XFREE */
 	case GDK_q:
 	case GDK_Q:
 		totem_action_exit (totem);
@@ -2870,6 +2896,18 @@ on_volume_scroll_event (GtkWidget *win, GdkEventScroll *event, Totem *totem)
 	return totem_action_handle_volume_scroll (totem, event->direction);
 }
 
+#ifdef HAVE_MEDIA_PLAYER_KEYS
+static gboolean
+on_window_focus_in_event (GtkWidget *win, GdkEventFocus *event, Totem *totem)
+{
+	if (totem->remote != NULL) {
+		totem_remote_window_activated (totem->remote);
+	}
+
+	return FALSE;
+}
+#endif
+
 static void
 update_media_menu_items (Totem *totem)
 {
@@ -3007,6 +3045,7 @@ totem_setup_window (Totem *totem)
 	g_signal_connect (G_OBJECT (main_pane), "size-allocate", G_CALLBACK (main_pane_size_allocated), totem);
 
 	totem_sidebar_setup (totem, show_sidebar, page_id);
+        g_free (page_id);
 }
 
 static void
@@ -3076,6 +3115,8 @@ totem_callback_connect (Totem *totem)
 	/* Screen size and Theme changes */
 	g_signal_connect (totem->win, "realize",
 			G_CALLBACK (window_realize_cb), totem);
+	g_signal_connect (totem->win, "unrealize",
+			G_CALLBACK (window_unrealize_cb), totem);
 
 	/* Motion notify for the Popups */
 	item = glade_xml_get_widget (totem->xml,
@@ -3176,6 +3217,14 @@ totem_callback_connect (Totem *totem)
 	totem_action_set_sensitivity ("play", FALSE);
 	totem_action_set_sensitivity ("next-chapter", FALSE);
 	totem_action_set_sensitivity ("previous-chapter", FALSE);
+	totem_action_set_sensitivity ("skip-forward", FALSE);
+	totem_action_set_sensitivity ("skip-backwards", FALSE);
+	totem_action_set_sensitivity ("skip-to", FALSE);
+
+#ifdef HAVE_MEDIA_PLAYER_KEYS
+	g_signal_connect (G_OBJECT(totem->win), "focus-in-event",
+			G_CALLBACK (on_window_focus_in_event), totem);
+#endif
 }
 
 static void
@@ -3215,6 +3264,9 @@ video_widget_create (Totem *totem)
 	GError *err = NULL;
 	GtkWidget *container;
 	BaconVideoWidget **bvw;
+	const GtkTargetEntry source_table[] = {
+		{ "text/uri-list", 0, 0 },
+	};
 
 	totem->scr = totem_scrsaver_new ();
 
@@ -3361,6 +3413,8 @@ main (int argc, char **argv)
 	GConfClient *gc;
 #ifndef HAVE_GTK_ONLY
 	GnomeProgram *program;
+#else
+	GError *error = NULL;
 #endif
 	GOptionContext *context;
 	GOptionGroup *baconoptiongroup;
@@ -3388,8 +3442,8 @@ main (int argc, char **argv)
 
 #ifdef HAVE_GTK_ONLY
 	g_option_context_add_group (context, gtk_get_option_group (TRUE));
-	if (g_option_context_parse (context, &argc, argv, &error) == FALSE) {
-		totem_action_error_and_exit (_("Totem could not parse the command-line options"), error->message);
+	if (g_option_context_parse (context, &argc, &argv, &error) == FALSE) {
+		totem_action_error_and_exit (_("Totem could not parse the command-line options"), error->message, NULL);
 	}
 #else
 	program = gnome_program_init (PACKAGE, VERSION,
